"use strict";
// Copyright 2021-2024 Buf Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPluginOptionError = exports.reasonToString = exports.PluginOptionError = void 0;
class PluginOptionError extends Error {
    constructor(option, reason) {
        const detail = reason !== undefined ? reasonToString(reason) : "";
        super(detail.length > 0
            ? `invalid option "${option}": ${detail}`
            : `invalid option "${option}"`);
        this.name = "PluginOptionError";
    }
}
exports.PluginOptionError = PluginOptionError;
function reasonToString(reason) {
    if (reason instanceof Error) {
        return reason.message;
    }
    if (typeof reason === "string") {
        return reason;
    }
    return String(reason);
}
exports.reasonToString = reasonToString;
function isPluginOptionError(arg) {
    if (!(arg instanceof Error)) {
        return false;
    }
    return arg.name === "PluginOptionError";
}
exports.isPluginOptionError = isPluginOptionError;
